@@ -3,7 +3,8 @@
 
 #pragma once
 
-#include "../console/terminal.hpp"
+#include "../desktopio/terminal.hpp"
+#include "../desktopio/application.hpp"
 
 namespace netxs::events::userland
 {
@@ -11,7 +12,7 @@ namespace netxs::events::userland
     {
         using mime = ansi::clip::mime;
 
-        EVENTPACK( term, netxs::events::userland::root::custom )
+        EVENTPACK( term, ui::e2::extra::slot3 )
         {
             EVENT_XS( cmd    , si32 ),
             GROUP_XS( preview, si32 ),
@@ -23,6 +24,7 @@ namespace netxs::events::userland
             {
                 EVENT_XS( align    , si32 ),
                 EVENT_XS( wrapln   , si32 ),
+                EVENT_XS( io_log   , bool ),
                 GROUP_XS( selection, si32 ),
                 GROUP_XS( colors   , rgba ),
 
@@ -41,6 +43,7 @@ namespace netxs::events::userland
             {
                 EVENT_XS( align    , si32 ),
                 EVENT_XS( wrapln   , si32 ),
+                EVENT_XS( io_log   , bool ),
                 GROUP_XS( selection, si32 ),
                 GROUP_XS( colors   , rgba ),
 
@@ -72,10 +75,12 @@ namespace netxs::events::userland
         };
     };
 }
-
 // term: Terminal Emulator.
 namespace netxs::app::term
 {
+    static constexpr auto id = "term";
+    static constexpr auto desc = "Desktopio Terminal";
+
     using events = netxs::events::userland::term;
     using mime = clip::mime;
 
@@ -88,11 +93,17 @@ namespace netxs::app::term
             if (boss.client)
             {
                 auto& item = *boss.client;
-                item.SIGNAL(tier::release, e2::data::text,              look.label);
+                item.SIGNAL(tier::release, e2::data::utf8,              look.label);
                 boss.SIGNAL(tier::preview, e2::form::prop::ui::tooltip, look.notes);
                 item.reflow();
             }
         }
+        static auto _update_gear(ui::pads& boss, menu::item& item, hids& gear)
+        {
+            auto& look = item.views[item.taken];
+            gear.set_tooltip(look.notes, true);
+            _update(boss, item);
+        }
         static auto _update_to(ui::pads& boss, menu::item& item, si32 i)
         {
             item.select(i);
@@ -104,20 +115,19 @@ namespace netxs::app::term
             if (item.brand == menu::item::Repeat)
             {
                 auto& tick = boss.plugins<pro::timer>();
-                boss.SUBMIT(tier::release, hids::events::mouse::button::down::left, gear)
+                boss.LISTEN(tier::release, hids::events::mouse::button::down::left, gear)
                 {
                     if (item.views.size())
                     {
                         item.taken = (item.taken + 1) % item.views.size();
-                        _update(boss, item);
                     }
                     if (gear.capture(boss.id))
                     {
                         proc(boss, item, gear);
-                        tick.actify(0, REPEAT_DELAY, [&, proc](auto p)
+                        tick.actify(0, skin::globals().repeat_delay, [&, proc](auto p)
                         {
                             proc(boss, item, gear);
-                            tick.actify(1, REPEAT_RATE, [&, proc](auto d)
+                            tick.actify(1, skin::globals().repeat_rate, [&, proc](auto d)
                             {
                                 proc(boss, item, gear);
                                 return true; // Repeat forever.
@@ -126,8 +136,12 @@ namespace netxs::app::term
                         });
                         gear.dismiss(true);
                     }
+                    if (item.views.size())
+                    {
+                        _update_gear(boss, item, gear);
+                    }
                 };
-                boss.SUBMIT(tier::release, hids::events::mouse::button::up::left, gear)
+                boss.LISTEN(tier::release, hids::events::mouse::button::up::left, gear)
                 {
                     tick.pacify();
                     gear.setfree();
@@ -135,10 +149,10 @@ namespace netxs::app::term
                     if (item.views.size() && item.taken)
                     {
                         item.taken = 0;
-                        _update(boss, item);
+                        _update_gear(boss, item, gear);
                     }
                 };
-                boss.SUBMIT(tier::release, e2::form::state::mouse, active)
+                boss.LISTEN(tier::release, e2::form::state::mouse, active)
                 {
                     if (!active && tick)
                     {
@@ -153,12 +167,12 @@ namespace netxs::app::term
             }
             else
             {
-                boss.SUBMIT(tier::release, hids::events::mouse::button::click::left, gear)
+                boss.LISTEN(tier::release, hids::events::mouse::button::click::left, gear)
                 {
                     proc(boss, item, gear);
                     if constexpr (AutoUpdate)
                     {
-                        if (item.brand == menu::item::Option) _update(boss, item);
+                        if (item.brand == menu::item::Option) _update_gear(boss, item, gear);
                     }
                     gear.dismiss(true);
                 };
@@ -166,7 +180,7 @@ namespace netxs::app::term
         };
     }
 
-    const auto terminal_menu = [](xml::settings& config)
+    const auto terminal_menu = [](xmls& config)
     {
         auto highlight_color = skin::color(tone::highlight);
         auto c3 = highlight_color;
@@ -183,10 +197,10 @@ namespace netxs::app::term
             { menu::type::Option,   menu::item::Option   },
             { menu::type::Repeat,   menu::item::Repeat   }};
 
-        #define PROC_LIST \
+        #define proc_list \
             X(Noop                      ) /* */ \
             X(TerminalQuit              ) /* */ \
-            X(TerminalMaximize          ) /* */ \
+            X(TerminalFullscreen        ) /* */ \
             X(TerminalRestart           ) /* */ \
             X(TerminalSendKey           ) /* */ \
             X(TerminalWrapMode          ) /* */ \
@@ -196,7 +210,8 @@ namespace netxs::app::term
             X(TerminalFindPrev          ) /* */ \
             X(TerminalUndo              ) /* Undo/Redo for cooked read under win32 */ \
             X(TerminalRedo              ) /* */ \
-            X(TerminalPaste             ) /* */ \
+            X(TerminalClipboardPaste    ) /* */ \
+            X(TerminalClipboardWipe     ) /* */ \
             X(TerminalSelectionCopy     ) /* */ \
             X(TerminalSelectionMode     ) /* */ \
             X(TerminalSelectionRect     ) /* Linear/Rectangular */ \
@@ -212,6 +227,7 @@ namespace netxs::app::term
             X(TerminalViewportTop       ) /* */ \
             X(TerminalViewportEnd       ) /* */ \
             X(TerminalViewportCopy      ) /* */ \
+            X(TerminalStdioLog          ) /* */ \
             X(TerminalLogStart          ) /* */ \
             X(TerminalLogPause          ) /* */ \
             X(TerminalLogStop           ) /* */ \
@@ -233,14 +249,14 @@ namespace netxs::app::term
         enum func
         {
             #define X(_proc) _proc,
-            PROC_LIST
+            proc_list
             #undef X
         };
 
         static const auto route_options = std::unordered_map<text, func>
         {
             #define X(_proc) { #_proc, func::_proc },
-            PROC_LIST
+            proc_list
             #undef X
         };
 
@@ -257,7 +273,7 @@ namespace netxs::app::term
                 {
                     boss.SIGNAL(tier::anycast, preview::wrapln, item.views[item.taken].value);
                 });
-                boss.SUBMIT(tier::anycast, release::wrapln, wrapln)
+                boss.LISTEN(tier::anycast, release::wrapln, wrapln)
                 {
                     _update_to(boss, item, wrapln);
                 };
@@ -269,7 +285,7 @@ namespace netxs::app::term
                 {
                     boss.SIGNAL(tier::anycast, preview::align, item.views[item.taken].value);
                 });
-                boss.SUBMIT(tier::anycast, release::align, align)
+                boss.LISTEN(tier::anycast, release::align, align)
                 {
                     _update_to(boss, item, align);
                 };
@@ -281,7 +297,7 @@ namespace netxs::app::term
                 {
                     boss.SIGNAL(tier::anycast, app::term::events::search::reverse, gear);
                 });
-                boss.SUBMIT(tier::anycast, app::term::events::search::status, status)
+                boss.LISTEN(tier::anycast, app::term::events::search::status, status)
                 {
                     _update_to(boss, item, (status & 2) ? 1 : 0);
                 };
@@ -293,7 +309,7 @@ namespace netxs::app::term
                 {
                     boss.SIGNAL(tier::anycast, app::term::events::search::forward, gear);
                 });
-                boss.SUBMIT(tier::anycast, app::term::events::search::status, status)
+                boss.LISTEN(tier::anycast, app::term::events::search::status, status)
                 {
                     _update_to(boss, item, (status & 1) ? 1 : 0);
                 };
@@ -316,14 +332,14 @@ namespace netxs::app::term
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    boss.base::template riseup<tier::release>(e2::form::quit, boss.This());
+                    boss.RISEUP(tier::release, e2::form::proceed::quit::one, boss.This());
                 });
             }
-            static void TerminalMaximize(ui::pads& boss, menu::item& item)
+            static void TerminalFullscreen(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    boss.base::template riseup<tier::release>(e2::form::maximize, gear);
+                    boss.RISEUP(tier::release, e2::form::layout::fullscreen, gear);
                 });
             }
             static void TerminalRestart(ui::pads& boss, menu::item& item)
@@ -347,13 +363,20 @@ namespace netxs::app::term
                     boss.SIGNAL(tier::anycast, app::term::events::cmd, ui::term::commands::ui::commands::redo);
                 });
             }
-            static void TerminalPaste(ui::pads& boss, menu::item& item)
+            static void TerminalClipboardPaste(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
                     boss.SIGNAL(tier::anycast, app::term::events::data::paste, gear);
                 });
             }
+            static void TerminalClipboardWipe(ui::pads& boss, menu::item& item)
+            {
+                _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
+                {
+                    gear.clear_clip_data();
+                });
+            }
             static void TerminalSelectionCopy(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
@@ -368,7 +391,7 @@ namespace netxs::app::term
                 {
                     boss.SIGNAL(tier::anycast, preview::selection::mode, item.views[item.taken].value);
                 });
-                boss.SUBMIT(tier::anycast, release::selection::mode, mode)
+                boss.LISTEN(tier::anycast, release::selection::mode, mode)
                 {
                     _update_to(boss, item, mode);
                 };
@@ -380,7 +403,7 @@ namespace netxs::app::term
                 {
                     boss.SIGNAL(tier::anycast, preview::selection::box, item.views[item.taken].value);
                 });
-                boss.SUBMIT(tier::anycast, release::selection::box, selbox)
+                boss.LISTEN(tier::anycast, release::selection::box, selbox)
                 {
                     _update_to(boss, item, selbox);
                 };
@@ -403,18 +426,14 @@ namespace netxs::app::term
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::y.param();
-                    info.vector = 1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info, ({ .vector = 1 }));
                 });
             }
             static void TerminalViewportPageDown(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::y.param();
-                    info.vector = -1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info, ({ .vector = -1 }));
                 });
             }
             static void TerminalViewportLineUp(ui::pads& boss, menu::item& item)
@@ -422,9 +441,7 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::y.param();
-                    info.vector = std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info, ({ .vector = std::abs(item.views[item.taken].value) }));
                 });
             }
             static void TerminalViewportLineDown(ui::pads& boss, menu::item& item)
@@ -432,43 +449,35 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::y.param();
-                    info.vector = -std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info, ({ .vector = -std::abs(item.views[item.taken].value) }));
                 });
             }
             static void TerminalViewportTop(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::to_top::y.param();
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_top::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_top::y, info, ());
                 });
             }
             static void TerminalViewportEnd(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::to_end::y.param();
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_end::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_end::y, info, ());
                 });
             }
             static void TerminalViewportPageLeft(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::x.param();
-                    info.vector = 1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info, ({ .vector = 1 }));
                 });
             }
             static void TerminalViewportPageRight(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::x.param();
-                    info.vector = -1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info, ({ .vector = -1 }));
                 });
             }
             static void TerminalViewportCharLeft(ui::pads& boss, menu::item& item)
@@ -476,9 +485,7 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::x.param();
-                    info.vector = std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info, ({ .vector = std::abs(item.views[item.taken].value) }));
                 });
             }
             static void TerminalViewportCharRight(ui::pads& boss, menu::item& item)
@@ -486,11 +493,21 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::x.param();
-                    info.vector = -std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info, ({ .vector = -std::abs(item.views[item.taken].value) }));
                 });
             }
+            static void TerminalStdioLog(ui::pads& boss, menu::item& item)
+            {
+                item.reindex([](auto& utf8){ return xml::take<bool>(utf8).value(); });
+                _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
+                {
+                    boss.SIGNAL(tier::anycast, preview::io_log, item.views[item.taken].value);
+                });
+                boss.LISTEN(tier::anycast, release::io_log, state)
+                {
+                    _update_to(boss, item, state);
+                };
+            }
             static void TerminalLogStart(ui::pads& boss, menu::item& item)
             {
 
@@ -564,16 +581,16 @@ namespace netxs::app::term
         static const auto proc_map = std::unordered_map<func, submit_proc>
         {
             #define X(_proc) { func::_proc, &disp::_proc },
-            PROC_LIST
+            proc_list
             #undef X
         };
-        #undef PROC_LIST
+        #undef proc_list
 
         auto list = menu::list{};
         auto defs = menu::item::look{};
         for (auto data_ptr : menudata)
         {
-            auto item_ptr = std::make_shared<menu::item>();
+            auto item_ptr = ptr::shared<menu::item>();
             auto& data = *data_ptr;
             auto& item = *item_ptr;
             auto route = data.take(menu::attr::route, func::Noop,          route_options);
@@ -594,14 +611,14 @@ namespace netxs::app::term
             }
             if (item.views.empty())
             {
-                log("term: drop menu item without label");
+                log(ansi::err(prompt::term, "Menu item without label"));
                 continue;
             }
             auto setup = [route](ui::pads& boss, menu::item& item)
             {
                 if (item.brand == menu::item::Option)
                 {
-                    boss.SUBMIT(tier::release, hids::events::mouse::button::click::left, gear)
+                    boss.LISTEN(tier::release, hids::events::mouse::button::click::left, gear)
                     {
                         item.taken = (item.taken + 1) % item.views.size();
                     };
@@ -616,7 +633,7 @@ namespace netxs::app::term
 
     namespace
     {
-        auto build = [](text cwd, text arg, xml::settings& config, text patch)
+        auto build = [](text cwd, text arg, xmls& config, text patch)
         {
             auto menu_white = skin::color(tone::menu_white);
             auto cB = menu_white;
@@ -627,77 +644,138 @@ namespace netxs::app::term
             arg = arg_shadow;
             if (term_type == shared::app_type::normal)
             {
+                //todo revise focus
                 window->plugin<pro::focus>()
                       ->plugin<pro::track>()
                       ->plugin<pro::acryl>()
                       ->plugin<pro::cache>();
             }
-            else window->plugin<pro::focus>(faux);
+            else window->plugin<pro::focus>(pro::focus::mode::focusable, faux);
 
             auto object = window->attach(ui::fork::ctor(axis::Y))
                                 ->colors(cB.fgc(), cB.bgc());
             auto term_stat_area = object->attach(slot::_2, ui::fork::ctor(axis::Y));
-                    auto layers = term_stat_area->attach(slot::_1, ui::cake::ctor())
-                                                ->plugin<pro::limit>(dot_11, twod{ 400,200 });
-                        auto scroll = layers->attach(ui::rail::ctor());
-                            auto min_size = twod{ 12,1 }; // mc crashes when window is too small
-                            auto max_size = -dot_11;
-                            auto forced_clamp = faux;
-                            auto forced_resize = true;
-                            scroll->plugin<pro::limit>(min_size, max_size, forced_clamp, forced_resize)
-                                  ->invoke([](auto& boss)
-                                  {
-                                    boss.SUBMIT(tier::preview, e2::form::prop::window::size, new_size)
-                                    {
-                                        // Axis x/y (see XTWINOPS):
-                                        //   -1 -- preserve
-                                        //    0 -- maximize (toggle)
-                                        if (new_size == dot_00) // Toggle maximize/restore terminal window (only if it is focused by someone).
-                                        {
-                                            auto gates = e2::form::state::keybd::enlist.param();
-                                            boss.SIGNAL(tier::anycast, e2::form::state::keybd::enlist, gates);
-                                            if (gates.size())
-                                            if (auto gate_ptr = bell::getref(gates.back()))
-                                            {
-                                                gate_ptr->SIGNAL(tier::release, e2::form::proceed::onbehalf, [&](auto& gear)
-                                                {
-                                                    boss.template riseup<tier::release>(e2::form::maximize, gear);
-                                                });
-                                            }
-                                        }
-                                        else
-                                        {
-                                            auto size = boss.size();
-                                            new_size = new_size.less(dot_11, size, std::max(dot_11, new_size));
-                                            boss.SIGNAL(tier::release, e2::form::prop::window::size, new_size);
-                                        }
-                                    };
-                                  });
+            auto layers = term_stat_area->attach(slot::_1, ui::cake::ctor())
+                                        ->plugin<pro::limit>(dot_11, twod{ 400,200 });
+            auto scroll = layers->attach(ui::rail::ctor());
+            auto min_size = twod{ 12,1 }; // mc crashes when window is too small
+            auto max_size = -dot_11;
+            auto forced_clamp = faux;
+            auto forced_resize = true;
+            scroll->plugin<pro::limit>(min_size, max_size, forced_clamp, forced_resize)
+                ->invoke([](auto& boss)
+                {
+                    boss.LISTEN(tier::preview, e2::form::prop::window::size, new_size)
+                    {
+                        // Axis x/y (see XTWINOPS):
+                        //   -1 -- preserve
+                        //    0 -- maximize (toggle)
+                        if (new_size == dot_00) // Toggle fullscreen terminal (only if it is focused by someone).
+                        {
+                            boss.RISEUP(tier::request, e2::form::state::keybd::enlist, gates, ());
+                            if (gates.size())
+                            if (auto gate_ptr = bell::getref(gates.back()))
+                            {
+                                gate_ptr->SIGNAL(tier::release, e2::form::proceed::onbehalf, [&](auto& gear)
+                                {
+                                    boss.RISEUP(tier::release, e2::form::layout::fullscreen, gear);
+                                });
+                            }
+                        }
+                        else
+                        {
+                            auto size = boss.size();
+                            new_size = new_size.less(dot_11, size, std::max(dot_11, new_size));
+                            boss.SIGNAL(tier::release, e2::form::prop::window::size, new_size);
+                        }
+                    };
+                });
 
-                            auto shell = os::env::shell() + " -i";
-                            auto inst = scroll->attach(ui::term::ctor(cwd, arg.empty() ? shell : arg, config));
+            auto shell = os::env::shell() + " -i";
+            auto inst = scroll->attach(ui::term::ctor(cwd, arg.empty() ? shell : arg, config))
+                              ->plugin<pro::focus>(pro::focus::mode::focused);
+            auto scroll_bars = layers->attach(ui::fork::ctor());
+            auto vt = scroll_bars->attach(slot::_2, ui::grip<axis::Y>::ctor(scroll));
+            static constexpr auto drawfx = [](auto& boss, auto& canvas, auto handle, auto object_len, auto handle_len, auto region_len, auto wide)
+            {
+                // Brightener isn't suitable for white backgrounds.
+                auto brighter = skin::color(tone::selector);
+                brighter.bga(std::min(0xFF, brighter.bga() * 3));
+                auto brighter_bgc = brighter.bgc();
+                auto boss_bgc = boss.base::color().bgc();
 
-                            auto scroll_bars = layers->attach(ui::fork::ctor());
-                                auto vt = scroll_bars->attach(slot::_2, ui::grip<axis::Y>::ctor(scroll));
-                                auto hz = term_stat_area->attach(slot::_2, ui::grip_fx2<axis::X>::ctor(scroll))
-                                                        ->plugin<pro::limit>(twod{ -1,1 }, twod{ -1,1 })
-                                                        ->invoke([&](auto& boss)
-                                                        {
-                                                            boss.SUBMIT(tier::anycast, app::term::events::release::colors::bg, bg)
-                                                            {
-                                                                boss.color(boss.color().bgc(bg).txt(""));
-                                                            };
-                                                        });
+                if (handle_len != region_len) // Show only if it is oversized.
+                {
+                    if (wide) // Draw full scrollbar on mouse hover
+                    {
+                        static auto box = ' ';
+                        canvas.fill([&](cell& c) { c.txt(box).link(boss.bell::id).xlight(); });
+                        canvas.fill(handle, [&](cell& c) { c.bgc().mix(brighter_bgc); });
+                    }
+                    else
+                    {
+                        static auto box = "▄"sv; //"▀"sv;
+                        canvas.fill(handle, [&](cell& c) { c.link(boss.bell::id).bgc().mix(brighter_bgc); });
+                        canvas.fill([&](cell& c) { c.inv(true).txt(box).fgc(boss_bgc); });
+                    }
+                }
+                else
+                {
+                    static auto box = "▄"sv;
+                    canvas.fill([&](cell& c) { c.inv(true).txt(box).fgc(boss_bgc); });
+                }
+            };
+            auto hz = term_stat_area->attach(slot::_2, ui::gripfx<axis::X, drawfx>::ctor(scroll))
+                ->plugin<pro::limit>(twod{ -1,1 }, twod{ -1,1 })
+                ->invoke([&](auto& boss)
+                {
+                    boss.LISTEN(tier::anycast, app::term::events::release::colors::bg, bg)
+                    {
+                        boss.color(boss.color().bgc(bg).txt(""));
+                    };
+                });
 
             auto [slot1, cover, menu_data] = terminal_menu(config);
             auto menu = object->attach(slot::_1, slot1);
-            auto menu_id = slot1->id;
-            cover->invoke([&](auto& boss)
+            cover->invoke([&, &slot1 = slot1](auto& boss) //todo clang 15.0.0 still disallows capturing structured bindings (wait for clang 16.0.0)
             {
-                boss.colors(cell{ cB }.inv(true).txt("▀"sv).link(menu_id));
-                boss.SUBMIT(tier::anycast, app::term::events::release::colors::bg, bg)
+                auto bar = cell{ "▀"sv }.link(slot1->id);
+                auto brush = ptr::shared(cell{ cB }.inv(true).txt("▀"sv).link(slot1->id));
+                auto winsz = ptr::shared(dot_00);
+                auto visible = ptr::shared(slot1->back() != boss.This());
+                auto check_state = ptr::function([state = testy<bool>{ true }, winsz, visible](base& boss) mutable
                 {
-                    boss.color(boss.color().fgc(bg));
+                    if (state(*visible || winsz->y != 1))
+                    {
+                        boss.RISEUP(tier::preview, e2::form::prop::ui::acryl, state.last);
+                        boss.RISEUP(tier::preview, e2::form::prop::ui::cache, state.last);
+                    }
+                });
+                boss.LISTEN(tier::release, e2::form::state::visible, menu_visible, -, (visible, check_state))
+                {
+                    *visible = menu_visible;
+                    (*check_state)(boss);
+                };
+                boss.LISTEN(tier::anycast, e2::form::upon::resize, newsize, -, (winsz, check_state))
+                {
+                    *winsz = newsize;
+                    (*check_state)(boss);
+                };
+                boss.LISTEN(tier::anycast, app::term::events::release::colors::bg, bg, -, (brush))
+                {
+                    brush->fgc(bg);
+                };
+                boss.LISTEN(tier::release, e2::render::any, parent_canvas, -, (bar, winsz, brush))
+                {
+                    if (winsz->y != 1)
+                    {
+                        auto& b = *brush;
+                        parent_canvas.fill([&](cell& c) { c.fusefull(b); });
+                    }
+                    else
+                    {
+                        parent_canvas.fill([&](cell& c) { c.fgc(c.bgc()).fga(0xff).bgc(0).txt(bar).link(bar); });
+                    }
                 };
             });
 
@@ -705,79 +783,93 @@ namespace netxs::app::term
                 ->attach_property(ui::term::events::colors::fg,      app::term::events::release::colors::fg)
                 ->attach_property(ui::term::events::selmod,          app::term::events::release::selection::mode)
                 ->attach_property(ui::term::events::selalt,          app::term::events::release::selection::box)
+                ->attach_property(ui::term::events::io_log,          app::term::events::release::io_log)
                 ->attach_property(ui::term::events::layout::wrapln,  app::term::events::release::wrapln)
                 ->attach_property(ui::term::events::layout::align,   app::term::events::release::align)
                 ->attach_property(ui::term::events::search::status,  app::term::events::search::status)
                 ->invoke([](auto& boss)
                 {
-                    boss.SUBMIT(tier::anycast, app::term::events::cmd, cmd)
+                    boss.LISTEN(tier::anycast, e2::form::proceed::quit::any, boss_ptr)
+                    {
+                        boss.SIGNAL(tier::preview, e2::form::proceed::quit::one, boss_ptr);
+                    };
+                    boss.LISTEN(tier::preview, e2::form::proceed::quit::one, item_ptr)
+                    {
+                        boss.stop();
+                        boss.RISEUP(tier::release, e2::form::proceed::quit::one, item_ptr);
+                    };
+                    boss.LISTEN(tier::anycast, app::term::events::cmd, cmd)
                     {
                         boss.exec_cmd(static_cast<ui::term::commands::ui::commands>(cmd));
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::data::in, data)
+                    boss.LISTEN(tier::anycast, app::term::events::data::in, data)
                     {
                         boss.data_in(data);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::data::out, data)
+                    boss.LISTEN(tier::anycast, app::term::events::data::out, data)
                     {
                         boss.data_out(data);
                     };
                     //todo add color picker to the menu
-                    boss.SUBMIT(tier::anycast, app::term::events::preview::colors::bg, bg)
+                    boss.LISTEN(tier::anycast, app::term::events::preview::colors::bg, bg)
                     {
                         boss.set_bg_color(bg);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::preview::colors::fg, fg)
+                    boss.LISTEN(tier::anycast, app::term::events::preview::colors::fg, fg)
                     {
                         boss.set_fg_color(fg);
                     };
-                    boss.SUBMIT(tier::anycast, e2::form::prop::colors::any, clr)
+                    boss.LISTEN(tier::anycast, e2::form::prop::colors::any, clr)
                     {
                         auto deed = boss.bell::template protos<tier::anycast>();
                              if (deed == e2::form::prop::colors::bg.id) boss.SIGNAL(tier::anycast, app::term::events::preview::colors::bg, clr);
                         else if (deed == e2::form::prop::colors::fg.id) boss.SIGNAL(tier::anycast, app::term::events::preview::colors::fg, clr);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::preview::selection::mode, selmod)
+                    boss.LISTEN(tier::anycast, app::term::events::preview::selection::mode, selmod)
                     {
                         boss.set_selmod(selmod);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::preview::selection::box, selbox)
+                    boss.LISTEN(tier::anycast, app::term::events::preview::selection::box, selbox)
                     {
                         boss.set_selalt(selbox);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::preview::wrapln, wrapln)
+                    boss.LISTEN(tier::anycast, app::term::events::preview::wrapln, wrapln)
                     {
                         boss.set_wrapln(wrapln);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::preview::align, align)
+                    boss.LISTEN(tier::anycast, app::term::events::preview::io_log, state)
+                    {
+                        boss.set_log(state);
+                    };
+                    boss.LISTEN(tier::anycast, app::term::events::preview::align, align)
                     {
                         boss.set_align(align);
                     };
-                    boss.SUBMIT(tier::anycast, e2::form::upon::started, root)
+                    boss.LISTEN(tier::anycast, e2::form::upon::started, root)
                     {
                         boss.start();
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::search::forward, gear)
+                    boss.LISTEN(tier::anycast, app::term::events::search::forward, gear)
                     {
                         boss.search(gear, feed::fwd);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::search::reverse, gear)
+                    boss.LISTEN(tier::anycast, app::term::events::search::reverse, gear)
                     {
                         boss.search(gear, feed::rev);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::data::paste, gear)
+                    boss.LISTEN(tier::anycast, app::term::events::data::paste, gear)
                     {
                         boss.paste(gear);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::data::copy, gear)
+                    boss.LISTEN(tier::anycast, app::term::events::data::copy, gear)
                     {
                         boss.copy(gear);
                     };
-                    boss.SUBMIT(tier::anycast, app::term::events::data::prnscrn, gear)
+                    boss.LISTEN(tier::anycast, app::term::events::data::prnscrn, gear)
                     {
                         boss.prnscrn(gear);
                     };
-                    boss.SUBMIT(tier::anycast, e2::form::upon::scroll::any, i)
+                    boss.LISTEN(tier::anycast, e2::form::upon::scroll::any, i)
                     {
                         auto info = e2::form::upon::scroll::bypage::y.param();
                         auto deed = boss.bell::template protos<tier::anycast>();
@@ -790,5 +882,5 @@ namespace netxs::app::term
         };
     }
 
-    app::shared::initialize builder{ "term", build };
+    app::shared::initialize builder{ app::term::id, build };
 }
\ No newline at end of file
